"""This file tests the `keep_recurrence_attributes` argument of `of` works as expected."""
import os

from datetime import datetime
from recurring_ical_events import of
from icalendar import Calendar

HERE = os.path.dirname(__name__) or "test"
CALENDARS_FOLDER = os.path.join(HERE, "calendars")
calendar_path = os.path.join(CALENDARS_FOLDER, "rdate.ics")


def test_keep_recurrence_attributes_default(calendars):
    with open(calendar_path, "rb") as file:
        content = file.read()

    calendar = Calendar.from_ical(content)
    today = datetime.today()
    one_year_ahead = today.replace(year=today.year + 1)

    events = of(calendar).between(today, one_year_ahead)
    for event in events:
        assert event.get("RRULE", False) is False
        assert event.get("RDATE", False) is False
        assert event.get("EXDATE", False) is False


def test_keep_recurrence_attributes_false(calendars):
    with open(calendar_path, "rb") as file:
        content = file.read()

    calendar = Calendar.from_ical(content)
    today = datetime.today()
    one_year_ahead = today.replace(year=today.year + 1)

    events = of(calendar, keep_recurrence_attributes=False).between(today, one_year_ahead)
    for event in events:
        assert event.get("RRULE", False) is False
        assert event.get("RDATE", False) is False
        assert event.get("EXDATE", False) is False


def test_keep_recurrence_attributes_true(calendars):
    with open(calendar_path, "rb") as file:
        content = file.read()

    calendar = Calendar.from_ical(content)
    today = datetime.today()
    one_year_ahead = today.replace(year=today.year + 1)

    events = of(calendar, keep_recurrence_attributes=True).between(today, one_year_ahead)
    for event in events:
        assert event.get("RRULE", False) is not False
        assert event.get("RDATE", False) is not False
        assert event.get("EXDATE", False) is not False



"""This file tests the overlap of events with the queried span.

It might be that events overlap the start and end of the time span.
This must test
- single events overlapping
- recurrent events overlapping
both
- start
- end
with both
- datetime
- date
"""

def test_implement_tests(todo):
    pass



'''
This tests the issue 20. Exdates seem to be ignored.
https://github.com/niccokunzmann/python-recurring-ical-events/issues/20
'''
import pytest

@pytest.mark.parametrize("exdate", 
    # exdates copied from the source
    "20191015T141500Z,20191022T141500Z,20191105T151500Z,20191119T151500Z,"
    "20191126T151500Z,20191203T151500Z,20191217T151500Z,20191224T151500Z,201912"
    "31T151500Z".split(","))
def test_exdates_do_not_show_up(exdate, calendars):
    """Test that certain exdates do not occur."""
    events = calendars.issue_20_exdate_ignored.at(exdate[:8])
    assert not events, "{} should not occur at {}.".format(
        events[0].to_ical().decode(),
        exdate)
    
def test_there_are_n_events(calendars):
    """Test the total numer of events."""
    assert len(calendars.issue_20_exdate_ignored.all()) == 8




"""Test that cancelled events are actually repeated as cancelled."""
import pytest

@pytest.mark.parametrize("date,attr,value", [
    ("20200128", "STATUS", None),
    ("20200129", "STATUS", "CANCELLED"),
    ("20200130", "STATUS", None),
    ("20200128", "TRANSP", "OPAQUE"),
    ("20200129", "TRANSP", "OPAQUE"),
    ("20200130", "TRANSP", "OPAQUE"),
])
def test_events_are_cancelles(calendars, date, attr, value):
    event = calendars.issue_18_cancel_status.at(date)[0]
    assert event.get(attr) == value




"""These tests are for Issue 48
https://github.com/niccokunzmann/python-recurring-ical-events/issues/48

These were the events occurring when the issue was raised:
    EVENT2:
    start: 2020-11-02 11:30:00+00:00
    stop:  2020-11-02 13:00:00+00:00

March to October: UTC+1
October to March: UTC+0

"""

import pytest
from pytz import timezone
from datetime import datetime, date

TZ = timezone("Europe/Lisbon")
@pytest.mark.parametrize("date,event_name",
        [(datetime(2020,11,2,11,15,0,0,TZ),0),
        (datetime(2020,11,2,11,31,0,0,TZ),"EVENT2"),
        (datetime(2020,11,2,12,0,0,0,TZ),"EVENT2"),
        (datetime(2020,11,2,12,1,0,0,TZ),"EVENT2"),
        (datetime(2020,11,2,12,59,0,0,TZ),"EVENT2"),
        (datetime(2020,11,2,13,1,0,0,TZ),0)
        ])
def test_event_timing(calendars,date,event_name):
    date = date.tzinfo.localize(date.replace(tzinfo= None))
    events = calendars.issue_48_daylight_aware_repeats.at(date)
    if event_name:
        assert len(events) == 1
        assert events[0]["UID"] == event_name
    else:
        assert not events, "no events expected"
